==> mrds_2.0.5.tar.gz: mrds/R/ddf.ds.R <==
#' CDS/MCDS Distance Detection Function Fitting
#' 
#' Fits a conventional distance sampling (CDS) (likelihood eq 6.6 in Laake and
#' Borchers 2004) or multi-covariate distance sampling (MCDS)(likelihood eq
#' 6.14 in Laake and Borchers 2004) model for the detection function of
#' observed distance data.  It only uses key functions and does not incorporate
#' adjustment functions as in CDS/MCDS analysis engines in DISTANCE (Marques
#' and Buckland 2004). Distance can be grouped (binned), ungrouped (unbinned)
#' or mixture of the two.  This function is not called directly by the user and
#' is called from \code{ddf},\code{ddf.io}, or \code{ddf.trial}.
#' 
#' For a complete description of each of the calling arguments, see
#' \code{\link{ddf}}.  The argument \code{model} in this function is the same
#' as \code{dsmodel} in \code{ddf}.  The argument \code{dataname} is the name
#' of the dataframe specified by the argument \code{data} in \code{ddf}. The
#' arguments \code{control},\code{meta.data},and \code{method} are defined the
#' same as in \code{ddf}.
#' 
#' @S3method ddf ds
#' @param model model list with key function and scale formula if any
#' @param data analysis dataframe
#' @param meta.data list containing settings controlling data structure
#' @param control list containing settings controlling model fitting
#' @param call original function call if this function not called directly from
#'   \code{ddf} (e.g., called via \code{ddf.io}
#' @param method analysis method; only needed if this function called from
#'   \code{ddf.io} or \code{ddf.trial}
#' @return result: a ds model object
#' @note If mixture of binned and unbinned distance, width must be set to be >=
#'   largest interval endpoint; this could be changed with a more complicated
#'   analysis; likewise, if all binned and bins overlap, the above must also
#'   hold; if bins don't overlap, width must be one of the interval endpoints;
#'   same holds for left truncation Although the mixture analysis works in
#'   principle it has not been tested via simulation.
#' @author Jeff Laake
#' @seealso \code{\link{flnl}},
#'   \code{\link{summary.ds}},\code{\link{coef.ds}},\code{\link{plot.ds}},\code{\link{gof.ds}}
#' @references Laake, J.L. and D.L. Borchers. 2004. Methods for incomplete
#'   detection at distance zero. In: Advanced Distance Sampling, eds. S.T.
#'   Buckland, D.R.Anderson, K.P. Burnham, J.L. Laake, D.L. Borchers, and L.
#'   Thomas. Oxford University Press.
#' 
#' Marques, F.F.C. and S.T. Buckland. 2004. Covariate models for the detection
#'   function. In: Advanced Distance Sampling, eds. S.T. Buckland,
#'   D.R.Anderson, K.P. Burnham, J.L. Laake, D.L. Borchers, and L. Thomas.
#'   Oxford University Press.
#' @keywords Statistical Models
#' @examples
#' 
#' #
#' # ddf.ds is called in the following example when ddf is called and method="ds"
#' #
#' data(book.tee.data)
#' region<<-book.tee.data$book.tee.region
#' egdata<<-book.tee.data$book.tee.dataframe
#' samples<<-book.tee.data$book.tee.samples
#' obs<<-book.tee.data$book.tee.obs
#' result=ddf(dsmodel = ~mcds(key = "hn", formula = ~1), data = egdata[egdata$observer == 
#'     1, ], method = "ds", meta.data = list(width = 4))
#' summary(result,se=TRUE)
#' plot(result,main="cds - observer 1")
#' print(dht(result,region,samples,obs,options=list(varflag=0,group=TRUE),se=TRUE))
#' print(ddf.gof(result))
#' 
#' 
ddf.ds <-function(model, data, meta.data=list(), control=list(),call,method="ds")
{
# 
# ddf.ds
#
# Fits distance sampling detection function for grouped (binned), ungrouped (unbinned) or mixture of the two.
#
# Note: if mixture, width must be set to be >= largest interval endpoint; this could be changed with
#       a more complicated analysis; likewise, if all binned and bins overlap, the above must also hold;
#       if bins don't overlap, width must be one of the interval endpoints; same holds for left truncation
#       Although the mixture analysis works in principle it has not been tested via simulation.
#

# Arguments:
#
#  model     - model list with key function, scale formula, shape formula
#  data      - dataframe
#  meta.data - list containing settings controlling data structure, selection
#  control   - list containing settings controlling model fitting
#

# Value:
#
#  result    - ds model object 
#
# Functions Used: 
#        assign.default.values, process.data, create.ddfobj, setbounds, detfct.fit, flt.var, 
#        predict(predict.ds), NCovered(NCovered.ds)
#
#
#   Code structure for optimization with optim
#
#    ddf.ds --> detfct.fit --> detfct.fit.opt --> optimx or solnp --> flnl 
#                   
#      flnl--> flt.lnl -->
#              fpt.lnl --> detfct
#                         tablecgf --> gstdint
#                         integratedetfct --> detfct, gstdint, tablecgf
#
#   Detection function and options are described in ddfobj which is created by create.ddfobj.
#   That function creates list structure and sets up initial values.
#
#  jll 18-Nov-04 added code to remove observations with missing distance
#  jll 2-June-05 need to allow for grouped data
#  dlm 6-July-05 commented out all logistic/p(0) stuff to start work with adjustment terms
#  dlm 31-May-06 started taking the optimisation stuff out to detfct.fit
#  dlm 9-July-07 added glue code and extras for Fortran optimisation with foptim
#  jll Aug-10    completely revamped code; foptim currently will not work without changes for ddfobj
#  dlm Oct-11 removing foptim, putting in Lorenzo's code...

#
# Set up meta data values
#
  setdoeachint=FALSE
  if(!is.null(meta.data$doeachint)) setdoeachint=TRUE

  meta.data<-assign.default.values(meta.data, left=0, width=NA, binned=FALSE, 
                                   int.range=NA, mono=FALSE, mono.strict=TRUE,
                                   point=FALSE)
#
# Set up control values
#
  control <- assign.default.values(control,showit=0,doeachint=FALSE,
                                estimate=TRUE,refit=TRUE,nrefits=25,initial=NA, 
                                lowerbounds=NA, upperbounds=NA, limit=TRUE,
                                parscale=NA, maxiter=12, standardize=TRUE, 
                                mono.points=20, mono.tol=1e-7, mono.delta=1e-7,
                                debug=FALSE,nofit=FALSE)

#
#  Save current user options and then set design contrasts to treatment style
#
  save.options<-options()
  options(contrasts=c("contr.treatment","contr.poly"))
#
#  Process data 
#   First remove data with missing distances
	if(!is.null(data$distance)){
		data <- data[!is.na(data$distance),]
   }else{
		data <- data[!is.na(data$distbegin)&!is.na(data$distend),]
   }
	if(is.null(data$object)){
     stop("\nobject field is missing in the data\n")
   }
#  Next call function to process data based on values of meta.data 
   datalist <- process.data(data,meta.data,check=FALSE)
   xmat <- datalist$xmat
   meta.data <- datalist$meta.data
#
#  31 March 06; added code to use all unique detections (observer=1) if observer is present
#
   if(!is.null(xmat$observer))
   {
      if(control$limit)
      {
         if(length(levels(factor(xmat$observer)))>1)
         {
            xmat <- xmat[xmat$observer==levels(factor(xmat$observer))[1],]
            xmat$detected <- rep(1,dim(xmat)[1])
         }
      }
   }
#
#  If the frame includes a variable detected, use only those with detected=1
#  fix: jll 18-Nov-04 - added braces so code would work as intended to add detected if missing
#
   if(!is.null(xmat$detected)){
     if(control$limit) xmat <- xmat[xmat$detected==1,]
   }else
     xmat$detected <- rep(1,dim(xmat)[1])
#
#  Make sure object #'s are unique
#
   if(length(unique(xmat$object))!=length(xmat$object)){
     stop("\nSome values of object field are duplicates.  They must be unique.\n")
   }
#
#  Setup default breaks
#
  if(meta.data$binned){
    breaks <- c(max(0,min(as.numeric(levels(as.factor(xmat$distbegin))))),
                as.numeric(levels(as.factor(xmat$distend))))
  }else{
    breaks <- NULL
  }
#
#  Setup detection model
#               
  ddfobj <- create.ddfobj(model,xmat,meta.data,control$initial) 
  if(!is.null(ddfobj$shape) && ncol(ddfobj$shape$dm)>1){
    doeachint <- TRUE
  }
  if(!setdoeachint & !is.null(ddfobj$shape)){
    control$doeachint <- TRUE
  }
  initialvalues <- c(ddfobj$shape$parameters,ddfobj$scale$parameters,
                     ddfobj$adjustment$parameters)
  bounds <- setbounds(control$lowerbounds,control$upperbounds,
                      initialvalues,ddfobj)
  misc.options<-list(point=meta.data$point, int.range=meta.data$int.range,
                     showit=control$showit, doeachint=control$doeachint,
                     integral.numeric=control$integral.numeric, breaks=breaks,
                     maxiter=control$maxiter, refit=control$refit,
                     nrefits=control$nrefits, parscale=control$parscale,
                     mono=meta.data$mono, mono.strict=meta.data$mono.strict,
                     binned=meta.data$binned, width=meta.data$width, 
                     standardize=control$standardize,
                     mono.points=control$mono.points,
                     mono.tol=control$mono.tol,
                     mono.delta=control$mono.delta,
                     debug=control$debug,nofit=control$nofit)

  # debug - print the initial values
  if(misc.options$showit>1){
    cat("initialvalues=",initialvalues,"\n")
  }


# Note there is a difference between maxit (the maximum numbr of iterations
# for optimx() uses) and maxiter (which is what detfct.fit uses.)
   optim.options <- list(maxit=300)

#
# Actually do the optimisation!
#
  lt <- detfct.fit(ddfobj,optim.options,bounds,misc.options)

#
# add call and others to return values
#
  result <- list(call=call,data=data[row.names(data)%in%row.names(xmat),],
              model=substitute(model),meta.data=meta.data, control=control,
              method=method,ds=lt,par=lt$par,lnl=-lt$value)
	  
  # if there was no convergence, return the fitting object incase it's useful
  # it won't be of the correct class or have the correct elements
  if(lt$converge!=0 & misc.options$debug){
    errors("No convergence, not calculating Hessian, predicted values, abundance")
    errors("Returned object is for debugging ONLY!")
    options(save.options)
    return(result)
  }

  # 4-Jan-12 dlm sometimes this fails, so wrap it up in a try()
  result$hessian <- try(flt.var(result$ds$aux$ddfobj, TCI = FALSE, misc.options))

  # 23-Jan-06 jll   Changed this back to use formula in Buckland et al or it 
  # doesn't match DISTANCE unless the result is singular
  if(class(result$hessian)=="try-error"){
    # the hessian returned from solnp() is not what we want, warn about that and don't
    # return it
    if(misc.options$mono){
      cat("First partial hessian calculation failed with monotonicity enforced, no hessian\n")
    }else{
      cat("First partial hessian calculation failed; using second-partial hessian\n")
      result$hessian <- lt$hessian
    }
  }else if(length(lt$par)>1){
    if(class(try(solve(result$hessian),silent=TRUE))=="try-error"){
      cat("First partial hessian is singular; using second-partial hessian\n")
      result$hessian <- lt$hessian
    }
  }
  modpaste <- paste(model)
  modelvalues <- try(eval(parse(text=modpaste[2:length(modpaste)])))
  class(result$ds) <- c(modelvalues$fct,"ds")
#
#   AIC computation
#
  n <- length(xmat$distance)
  npar <- length(lt$par)
  result$criterion <- 2*lt$value + 2*npar

  class(result) <- c("ds","ddf")
  
  if(is.null(lt$message)){
    result$ds$message <- ""
  }

  if(lt$message == "FALSE CONVERGENCE"){
    errors("Model fitting did not converge.  Try different initial values or different model")
  }else{
    result$fitted <- predict(result,esw=FALSE)$fitted
    if(control$estimate){
      result$Nhat <- NCovered(result,group=TRUE)
    }
  }
#
# Restore user options
#
  options(save.options)
#

# Return result
#
  return(result)
}

==> mrds_2.0.9.tar.gz: mrds/R/ddf.ds.R <==
#' CDS/MCDS Distance Detection Function Fitting
#' 
#' Fits a conventional distance sampling (CDS) (likelihood eq 6.6 in Laake and
#' Borchers 2004) or multi-covariate distance sampling (MCDS)(likelihood eq
#' 6.14 in Laake and Borchers 2004) model for the detection function of
#' observed distance data.  It only uses key functions and does not incorporate
#' adjustment functions as in CDS/MCDS analysis engines in DISTANCE (Marques
#' and Buckland 2004). Distance can be grouped (binned), ungrouped (unbinned)
#' or mixture of the two.  This function is not called directly by the user and
#' is called from \code{ddf},\code{ddf.io}, or \code{ddf.trial}.
#' 
#' For a complete description of each of the calling arguments, see
#' \code{\link{ddf}}.  The argument \code{model} in this function is the same
#' as \code{dsmodel} in \code{ddf}.  The argument \code{dataname} is the name
#' of the dataframe specified by the argument \code{data} in \code{ddf}. The
#' arguments \code{control},\code{meta.data},and \code{method} are defined the
#' same as in \code{ddf}.
#' 
#' @S3method ddf ds
#' @param model model list with key function and scale formula if any
#' @param data analysis dataframe
#' @param meta.data list containing settings controlling data structure
#' @param control list containing settings controlling model fitting
#' @param call original function call if this function not called directly from
#'   \code{ddf} (e.g., called via \code{ddf.io}
#' @param method analysis method; only needed if this function called from
#'   \code{ddf.io} or \code{ddf.trial}
#' @return result: a ds model object
#' @note If mixture of binned and unbinned distance, width must be set to be >=
#'   largest interval endpoint; this could be changed with a more complicated
#'   analysis; likewise, if all binned and bins overlap, the above must also
#'   hold; if bins don't overlap, width must be one of the interval endpoints;
#'   same holds for left truncation Although the mixture analysis works in
#'   principle it has not been tested via simulation.
#' @author Jeff Laake
#' @seealso \code{\link{flnl}},
#'   \code{\link{summary.ds}},\code{\link{coef.ds}},\code{\link{plot.ds}},\code{\link{gof.ds}}
#' @references Laake, J.L. and D.L. Borchers. 2004. Methods for incomplete
#'   detection at distance zero. In: Advanced Distance Sampling, eds. S.T.
#'   Buckland, D.R.Anderson, K.P. Burnham, J.L. Laake, D.L. Borchers, and L.
#'   Thomas. Oxford University Press.
#' 
#' Marques, F.F.C. and S.T. Buckland. 2004. Covariate models for the detection
#'   function. In: Advanced Distance Sampling, eds. S.T. Buckland,
#'   D.R.Anderson, K.P. Burnham, J.L. Laake, D.L. Borchers, and L. Thomas.
#'   Oxford University Press.
#' @keywords Statistical Models
#' @examples
#' 
#' 
#' # ddf.ds is called when ddf is called with method="ds"
#' \donttest{
#' data(book.tee.data)
#' region<<-book.tee.data$book.tee.region
#' egdata<<-book.tee.data$book.tee.dataframe
#' samples<<-book.tee.data$book.tee.samples
#' obs<<-book.tee.data$book.tee.obs
#' result=ddf(dsmodel = ~mcds(key = "hn", formula = ~1), data = egdata[egdata$observer == 
#'     1, ], method = "ds", meta.data = list(width = 4))
#' summary(result,se=TRUE)
#' plot(result,main="cds - observer 1")
#' print(dht(result,region,samples,obs,options=list(varflag=0,group=TRUE),se=TRUE))
#' print(ddf.gof(result))
#' }
#' 
ddf.ds <-function(model, data, meta.data=list(), control=list(),call,method="ds")
{
# Functions Used: 
#        assign.default.values, process.data, create.ddfobj, setbounds, 
#        detfct.fit, flt.var, predict(predict.ds), NCovered(NCovered.ds)
#
#   Code structure for optimization with optim
#
#    ddf.ds --> detfct.fit --> detfct.fit.opt --> optimx or solnp --> flnl 
#                   
#      flnl--> flpt.lnl --> distpdf ---> detfct
#                           tablecgf --> gstdint --> integratepdf ---> distpdf
#                           integratedpdf --> distpdf
#
#   Detection function and options are described in ddfobj which is created by create.ddfobj.
#   That function creates list structure and sets up initial values.
#
#  jll 18-Nov-04 added code to remove observations with missing distance
#  jll 2-June-05 need to allow for grouped data
#  dlm 6-July-05 commented out all logistic/p(0) stuff to start work with adjustment terms
#  dlm 31-May-06 started taking the optimisation stuff out to detfct.fit
#  dlm 9-July-07 added glue code and extras for Fortran optimisation with foptim
#  jll Aug-10    completely revamped code; foptim currently will not work without changes for ddfobj
#  dlm Oct-11 removing foptim, putting in Lorenzo's code...

#
# Set up meta data values
#
  meta.data<-assign.default.values(meta.data, left=0, width=NA, binned=FALSE, 
                                   int.range=NA, mono=FALSE, mono.strict=TRUE,
                                   point=FALSE)
#
# Set up control values
#
  control <- assign.default.values(control,showit=0,doeachint=FALSE,
                                estimate=TRUE,refit=TRUE,nrefits=25,initial=NA, 
                                lowerbounds=NA, upperbounds=NA, limit=TRUE,
                                parscale=NA, maxiter=12, standardize=TRUE, 
                                mono.points=20, mono.tol=1e-7, mono.delta=1e-7,
                                debug=FALSE,nofit=FALSE,optimx.method="nlminb",
                                optimx.maxit=300)

#
#  Save current user options and then set design contrasts to treatment style
#
  save.options<-options()
  options(contrasts=c("contr.treatment","contr.poly"))
#
#  Process data 
#   First remove data with missing distances
	if(!is.null(data$distance)){
		data <- data[!is.na(data$distance),]
   }else{
		data <- data[!is.na(data$distbegin)&!is.na(data$distend),]
   }
	if(is.null(data$object)){
     stop("\nobject field is missing in the data\n")
   }
#  Next call function to process data based on values of meta.data 
   datalist <- process.data(data,meta.data,check=FALSE)
   xmat <- datalist$xmat
   meta.data <- datalist$meta.data
#
#  31 March 06; added code to use all unique detections (observer=1) if observer is present
#
   if(!is.null(xmat$observer)){
      if(control$limit){
         if(length(levels(factor(xmat$observer)))>1){
            xmat <- xmat[xmat$observer==levels(factor(xmat$observer))[1],]
            xmat$detected <- rep(1,dim(xmat)[1])
         }
      }
   }
#
#  If the frame includes a variable detected, use only those with detected=1
#  fix: jll 18-Nov-04 - added braces so code would work as intended to add detected if missing
#
   if(!is.null(xmat$detected)){
     if(control$limit) xmat <- xmat[xmat$detected==1,]
   }else
     xmat$detected <- rep(1,dim(xmat)[1])
#
#  Make sure object #'s are unique
#
   if(length(unique(xmat$object))!=length(xmat$object)){
     stop("\nSome values of object field are duplicates.  They must be unique.\n")
   }
#
#  Setup default breaks
#
  if(meta.data$binned){
    breaks <- c(max(0,min(as.numeric(levels(as.factor(xmat$distbegin))))),
                as.numeric(levels(as.factor(xmat$distend))))
  }else{
    breaks <- NULL
  }
#
#  Setup detection model
#               
  ddfobj <- create.ddfobj(model,xmat,meta.data,control$initial) 
  # set doeachint=TRUE if a shape formula is used
  if(!is.null(ddfobj$shape) && ncol(ddfobj$shape$dm)>1){
    control$doeachint <- TRUE
  }
  # set doeachint=TRUE if adj.scale="width" and key not uniform
  if(ddfobj$type!="unif"&&!is.null(ddfobj$adjustment))
	  if(ddfobj$adjustment$scale=="width")
	  {
	    # setting doeachint to TRUE; 
		# cannot use integral scaling with adj.scale=width and non-uniform key
		  control$doeachint=TRUE
	  }
  initialvalues <- c(ddfobj$shape$parameters,ddfobj$scale$parameters,
                     ddfobj$adjustment$parameters)
  if(!is.null(initialvalues))
     bounds <- setbounds(control$lowerbounds,control$upperbounds,
                      initialvalues,ddfobj)
  else
	 bounds <- NULL
  misc.options<-list(point=meta.data$point, int.range=meta.data$int.range,
                     showit=control$showit, doeachint=control$doeachint,
                     integral.numeric=control$integral.numeric, breaks=breaks,
                     maxiter=control$maxiter, refit=control$refit,
                     nrefits=control$nrefits, parscale=control$parscale,
                     mono=meta.data$mono, mono.strict=meta.data$mono.strict,
                     binned=meta.data$binned, width=meta.data$width, 
                     standardize=control$standardize,
                     mono.points=control$mono.points,
                     mono.tol=control$mono.tol,
                     mono.delta=control$mono.delta,
                     debug=control$debug,nofit=control$nofit
                    )
  # debug - print the initial values
  if(misc.options$showit>1 && !is.null(initialvalues)){
    cat("initialvalues=",initialvalues,"\n")
  }


  # Note there is a difference between maxit (the maximum numbr of iterations
  # for optimx() uses) and maxiter (which is what detfct.fit uses.)
  optim.options <- list(maxit=control$optimx.maxit,
                        optimx.method=control$optimx.method)

#
# Actually do the optimisation if not just a uniform key!
#
  if(is.null(initialvalues))misc.options$nofit <- TRUE
  lt <- detfct.fit(ddfobj,optim.options,bounds,misc.options)
#  else
#	  lt <- list(par=NULL,value=sum(flpt.lnl(fpar=NULL,ddfobj,misc.options)),converge=0,message="")	  
#  }
#
# add call and others to return values
#
  stored_data=data[row.names(data)%in%row.names(xmat),]
  stored_data$detected=1
  result <- list(call=call,data=stored_data,
              model=substitute(model),meta.data=meta.data, control=control,
              method=method,ds=lt,par=lt$par,lnl=-lt$value)
	  
  # if there was no convergence, return the fitting object incase it's useful
  # it won't be of the correct class or have the correct elements
  if(lt$converge!=0 & misc.options$debug){
    errors("No convergence, not calculating Hessian, predicted values, abundance")
    errors("Returned object is for debugging ONLY!")
    options(save.options)
    return(result)
  }

  # 4-Jan-12 dlm sometimes this fails, so wrap it up in a try()
  if(is.null(lt$par))
     lt$hessian <- NULL	  
  else
  {	  
	  result$hessian <- try(flt.var(result$ds$aux$ddfobj, misc.options))
  # 23-Jan-06 jll   Changed this back to use formula in Buckland et al or it 
  # doesn't match DISTANCE unless the result is singular
     if(class(result$hessian)=="try-error"){
    # the hessian returned from solnp() is not what we want, warn about 
    # that and don't return it
         if(misc.options$mono){
            cat("First partial hessian calculation failed with monotonicity enforced, no hessian\n")
         }else{
            cat("First partial hessian calculation failed; using second-partial hessian\n")
            result$hessian <- lt$hessian
         }
    }else if(length(lt$par)>1){
       if(class(try(solve(result$hessian),silent=TRUE))=="try-error"){
           cat("First partial hessian is singular; using second-partial hessian\n")
           result$hessian <- lt$hessian
       }
    }
  }
  modpaste <- paste(model)
  modelvalues <- try(eval(parse(text=modpaste[2:length(modpaste)])))
  class(result$ds) <- c(modelvalues$fct,"ds")

  result$dsmodel <- modpaste

  # AIC computation
  n <- length(xmat$distance)
  npar <- length(lt$par)
  result$criterion <- 2*lt$value + 2*npar

  class(result) <- c("ds","ddf")
  
  if(is.null(lt$message)){
    result$ds$message <- ""
  }

  if(lt$message == "FALSE CONVERGENCE"){
    errors("Model fitting did not converge.  Try different initial values or different model")
  }else{
    result$fitted <- predict(result,esw=FALSE)$fitted
    if(control$estimate){
      result$Nhat <- NCovered(result,group=TRUE)
    }
  }
#
# Restore user options
#
  options(save.options)
#

# Return result
#
  return(result)
}
